// Copyright (C) 2014 - Scilab-Enterprises
// This file is released under the 3-clause BSD license. See COPYING-BSD.

function labyrinthDemo(f)
    // Design GUI
    ctrlFrame = uicontrol("parent", f, ...
    "style", "frame", ...
    "constraints", createConstraints("border", "left"),...
    "layout", "gridbag");

    btnFrame = uicontrol("parent", ctrlFrame, ...
    "style", "frame", ...
    "constraints", createConstraints("gridbag", [1,1,1,1], [0,0], "both"),...
    "layout", "gridbag", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "Actions"));
    btnLoadImage= uicontrol("parent", btnFrame, ...
    "style", "pushbutton", ...
    "constraints", createConstraints("gridbag", [1,1,1,1], [0,0], "both"), ...
    "string", "Load image", ...
    "icon", "plot", ...
    "callback", "cbLoadImage");
    btnTTTImage= uicontrol("parent", btnFrame, ...
    "style", "pushbutton", ...
    "constraints", createConstraints("gridbag", [1,2,1,1], [0,0], "both"), ...
    "string", "Decompose image", ...
    "icon", "preferences-system", ...
    "callback", "cbDecomposeImage");
    btnExplorePath= uicontrol("parent", btnFrame, ...
    "style", "pushbutton", ...
    "constraints", createConstraints("gridbag", [1,3,1,1], [0,0], "both"), ...
    "string", "Explore path", ...
    "icon", "applications-system", ...
    "callback", "cbExplorePath");

    visibleFrame = uicontrol("parent", ctrlFrame, ...
    "style", "frame",...
    "constraints", createConstraints("gridbag", [1,2,1,1], [0,0], "both"),...
    "layout", "gridbag", ...
    "tag", "visibleFrame", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "Display"));
    uicontrol("parent", visibleFrame, ...
    "style", "checkbox", ...
    "string", "Original image", ...
    "callback", "cbSetVisible(''originalMatplot'', ''originalVisible'')",...
    "constraints", createConstraints("gridbag", [1,1,1,1],[0,0], "both"), ...
    "value", 1, ...
    "tag", "originalVisible", ...
    "enable", "off")
    uicontrol("parent", visibleFrame, ...
    "style", "checkbox", ...
    "string", "Shortest path", ...
    "callback", "cbSetVisible(''shortestPath'', ''shortestPathVisible'')",...
    "constraints", createConstraints("gridbag", [1,3,1,1],[0,0], "both"), ...
    "value", 1, ...
    "tag", "shortestPathVisible", ...
    "enable", "off")
    uicontrol("parent", visibleFrame, ...
    "style", "checkbox", ...
    "string", "Scilab image", ...
    "callback", "cbSetVisible(''workingMatplot'', ''workingVisible'')",...
    "constraints", createConstraints("gridbag", [1,2,1,1],[0,0], "both"), ...
    "value", 1, ...
    "tag", "workingVisible", ...
    "enable", "off")

    graphicFrame = uicontrol("parent", f, ...
    "style", "frame", ...
    "constraints", createConstraints("border", "center"));
    a = newaxes(graphicFrame);
    a.tag="labAxes";
endfunction

function cbSetVisible(graphicTag, uiCtrlTag)
    if get(uiCtrlTag, "value") == 1 then
        set(graphicTag, "visible", "on");
    else
        set(graphicTag, "visible", "off");
    end
endfunction

function cbLoadImage()
    global data;
    jimport javax.imageio.ImageIO;
    jimport java.net.URL;

    bi = ImageIO.read(URL.new("file:////"+pwd()+"/lab.jpg"));
    w = bi.getWidth(jvoid);
    h = bi.getHeight(jvoid);
    data = bi.getRGB(0, 0, w, h, [], 0, w);
    data = matrix(data, double(w), double(h));
    data = data';

    // invertImage
    data = data(:, $:-1:1);

    // limit data
    data = data(2:$, 10:$);
    sca(get("labAxes"));
    a=gca();
    Matplot(int32(data));
    e=gce();
    set(e, "tag", "originalMatplot");

    set(get("originalVisible"), "enable", "on")
    a.axes_visible="off"
endfunction

function cbDecomposeImage()
    global data
    global dataToWork

    set(get("workingVisible"), "enable", "on")
    global bigPixel;
    bigPixel = 3;

    r = size(data, 'r') / bigPixel;
    c = size(data, 'c') / bigPixel;
    tmpData = zeros(data);
    tmpData(uint8(data) > 200) = -2;

    dataToWork = -2 * ones(r, c);
    for i =0:r-1
        for j=0:c-1
            if or(tmpData(bigPixel * i + [1:2], bigPixel * j+[1:bigPixel]) <> -2) then
                dataToWork(i+1, j+1) = 0;
            end
        end
    end
    f=gcf();
    f.color_map = coolcolormap(1900);
    sca(get("labAxes"));
    a = gca();
    Matplot(dataToWork)
    e = gce();
    set(e, "tag", "workingMatplot");
    set(e, "rect", get("originalMatplot", "rect"));
    a.axes_visible="off"
endfunction

function cbExplorePath()
    global dataToWork;
    global bigPixel;

    xstart = floor(92 / bigPixel);
    ystart = floor(98 / bigPixel);
    dataToWork(xstart,ystart) = 1;
    tic();
    [dataToWork] = findPathBFS(dataToWork,xstart, ystart);

    m = max(dataToWork);
    [y, x] = find(dataToWork == m);
    allx = max(x);
    ally = max(y);
    for i = m-1:-1:1
        [y, x] = find(dataToWork == i);
        d = sqrt(abs(y - ally($)).^2 + abs(x - allx($)).^2);
        [dmin, k] = min(d);
        allx($+1) = x(k);
        ally($+1) = y(k);
    end
    allx = allx .* bigPixel
    ally = (size(dataToWork, 'r') - ally) .* bigPixel
    sca(get("labAxes"));

    plot(allx, ally + 1, "g");
    e=gce();
    e.children(1).thickness = 5;
    e.children(1).tag="shortestPath";


    set(get("shortestPathVisible"), "enable", "on")
    a=gca();
    a.axes_visible = "off";

    xinfo("[BFS] Done in "+string(toc())+"s !!");
endfunction

function [M] = findPathBFS(M, xstart, ystart)
    for i = 1:size(M,'*')
        [x,y] = find(M == i);
        z = x + y;
        indx = x < xstart;
        indy = y < ystart;
        ind = ~(indx & indy);
        x = x(ind);
        y = y(ind);

        if or(x == size(M,'r') - 1) then
            break;
        end
        if isempty(x) then
            break;
        end

        M = exploreBFS(M, x, y)
        updateDisplay(M)

    end
endfunction

function M = exploreBFS(M, x, y)
    step = 1;
    for i = 1:size(x,'*')
        M = updateBFS(M, x(i), y(i), step, 0);
        M = updateBFS(M, x(i), y(i), 0, step);
        M = updateBFS(M, x(i), y(i), -step, 0);
        M = updateBFS(M, x(i), y(i), 0, -step);
    end
endfunction

function M = updateBFS(M, x, y, x_move, y_move)
    if x + x_move > 0 & x + x_move <= size(M, 'r') & ...
        y + y_move > 0 & y + y_move <= size(M, 'c') & ...
        (M(x+x_move, y+y_move) == -2 | M(x + x_move, y + y_move) > M(x, y) + 1) then
        M(x + x_move, y + y_move) = M(x, y) + 1;
    end
endfunction

function updateDisplay(M)
    e = findobj("tag", "workingMatplot");
    e.data = M;
endfunction
