// Copyright (C) 2014 - Scilab-Enterprises
// This file is released under the 3-clause BSD license. See COPYING-BSD.

mode(-1);

//==============================================================================
// Figure
//==============================================================================
delete(findobj("tag", "scilab550video"));
f = figure( ...
"dockable", "off", ...
"infobar_visible", "off", ...
"toolbar_visible", "off", ...
"menubar_visible", "off", ...
"default_axes", "off",...
"visible", "off");
f.tag = "scilab550video";
f.figure_size = [800 600];
f.layout = "border";
f.figure_name = "Scilab 5.5.0 new features";
f.icon = "applications-system";
f.background = -2;

//==============================================================================
// Global tab
//==============================================================================
tab = uicontrol("parent", f, "style", "tab");

//==============================================================================
// PathFinder Tab
//==============================================================================
exec("./Labyrinth.sce");
matplotDemo = uicontrol("parent", tab, ...
"style", "frame", ...
"string", "Matplot", ...
"layout", "gridbag");
matplotFrame = uicontrol("parent", matplotDemo, ...
"style", "frame", ...
"constraints", createConstraints("gridbag", [1, 1, 1 1], [1 1], "both"), ...
"layout", "border");

labyrinthDemo(matplotFrame);

//==============================================================================
// Xcos tab
//==============================================================================
function xcosTab()
    xcosTab = uicontrol("parent", tab, ...
    "style", "frame", ...
    "string", "Xcos", ...
    "layout", "gridbag");
    xcosFrame = uicontrol("parent", xcosTab, ...
    "style", "frame",...
    "layout", "gridbag");
    uicontrol("parent", xcosFrame, ...
    "style", "image", ...
    "string", "pendulum.png", ...
    "constraints", createConstraints("gridbag", [1, 1, 1 1], [1 1], "both"));
    uicontrol("parent", xcosFrame, ...
    "string", "Launch Xcos demonstration", ...
    "icon", "utilities-system-monitor", ...
    "callback", "exec(""setw.sci"",-1);xcos(""Inverted_pendulum.zcos"");", ...
    "constraints", createConstraints("gridbag", [1, 2, 1 1], [1 1], "both"));
endfunction
xcosTab();

//==============================================================================
// Buttons tab
//==============================================================================
function buttonsWithIcons()
    buttonsTab = uicontrol("parent", tab, ...
    "style", "frame", ...
    "layout", "gridbag", ...
    "string", "Buttons with icons", ...
    "tag", "buttonstab");

    createParts = %F;
    if createParts then
        // Image loading and splitting using external objects Java
        jimport javax.imageio.ImageIO;
        jimport java.net.URL;
        jimport java.io.File;

        bi = ImageIO.read(URL.new("file:////" + pwd() + "/scilab-square-logo.png"));
        for kR=1:3
            for kC=1:3
                subbi = bi.getSubimage(uint32((kR-1)*100), uint32(((kC-1)*100)), uint32(100), uint32(100));
                ImageIO.write(subbi, "png", File.new("logopart" + string(kR) + "_" + string(kC) + ".png"))
            end
        end
    end

    // Grid creation
    layoutOpts =  createLayoutOptions("gridbag");
    frame = uicontrol("parent", buttonsTab, ...
    "style", "frame", ...
    "backgroundcolor", [1 1 1], ...
    "layout", "gridbag", ...
    "layout_options", layoutOpts, ..
    "border", createBorder("titled", createBorder("etched", "lowered"), "Scilab logo"), ...
    "tag", "playframe");

    for kR=3:-1:1
        for kC=3:-1:1
            if (kR==1 & kC==1) then
                // Label for empty space
                uicontrol("parent", frame, ...
                "style", "text", ...
                "constraints", createConstraints("gridbag", [1, 1, 1 1]), ...
                "tag", "empty");
            else // Buttons
                uicontrol("parent", frame, ...
                "style", "pushbutton", ...
                "constraints", createConstraints("gridbag", [kC, kR, 1 1]), ...
                "icon", "logopart" + string(kC) + "_" + string(kR) + ".png", ...
                "callback", "moveButton();");
            end
        end
    end
endfunction
buttonsWithIcons();

// Callback for buttons
function moveButton
    btnConstraints = gcbo.constraints;
    emptyConstraints = get("empty", "constraints");
    buttonGrid = btnConstraints.grid(1:2);
    emptyGrid = emptyConstraints.grid(1:2);
    gridDiff = abs(buttonGrid-emptyGrid);
    if size(find(gridDiff==1), "*")==1 & size(find(gridDiff==0), "*")==1 then
        set(gcbo, "visible", "off");
        set(gcbo, "constraints", emptyConstraints);
        set(gcbo, "parent", findobj("tag", "buttonstab"));
        set(gcbo, "parent", findobj("tag", "playframe"));
        set(gcbo, "visible", "on");
        set("empty", "visible", "off");
        set("empty", "constraints", btnConstraints);
        set("empty", "parent", findobj("tag", "buttonstab"));
        set("empty", "parent", findobj("tag", "playframe"));
        set("empty", "visible", "on");
    end
endfunction


function launchPlot2dDemo()
    a = get("plot2d");
    sca(a);
    plot2d();
    popup1 = get("popup1");
    popup2 = get("popup2");
    popup3 = get("popup3");
    popup1.value = a.children(1).children(1).foreground;
    popup2.value = abs(a.children(1).children(2).mark_foreground);
    popup3.value = abs(a.children(1).children(3).mark_foreground);
    a.title.text = "plot3d demonstration";
    a.title.font_size = 5;
endfunction
//==============================================================================
// Plot interaction tab
//==============================================================================
function plot2dInterations()
    plotTab = uicontrol("parent", tab, ...
    "style", "frame", ...
    "layout", "border", ...
    "string", "Plot interaction");

    btnFrame = uicontrol("parent", plotTab, ...
    "style", "frame", ...
    "constraints", createConstraints("border", "left", [250 0]), ...
    "layout", "gridbag");
    btnLaunch = uicontrol("parent", btnFrame, ...
    "style", "pushbutton", ...
    "string", "Plot2d Demo", ...
    "icon", "plot", ...
    "constraints", createConstraints("gridbag", [1,1,1,1], [0,0], "both"), ...
    "callback", "launchPlot2dDemo");

    // Generate colors for popup
    cmap = f.color_map*255;
    colors = [];
    for k=1:size(cmap, 1)
        colors = [colors; msprintf("#%02x%02x%02x", cmap(k, 1), cmap(k, 2), cmap(k, 3)) "Index #" + string(k) + " in colormap"];
    end
    frame = uicontrol("parent", btnFrame, ...
    "style", "frame", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "Polyline #1"), ...
    "constraints", createConstraints("gridbag", [1, 2, 1, 1]), ...
    "layout", "border");
    popup1 = uicontrol("parent", frame, ...
    "style", "popupmenu", ...
    "string", colors, ...
    "callback", "a=get(""plot2d"");a.children(1).children(1).foreground = gcbo.value;", ...
    "tag", "popup1");
    frame = uicontrol("parent", btnFrame, ...
    "style", "frame", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "Polyline #2"), ...
    "constraints", createConstraints("gridbag", [1, 3, 1, 1]), ...
    "layout", "border");
    popup2 = uicontrol("parent", frame, ...
    "style", "popupmenu", ...
    "string", colors, ...
    "callback", "a=get(""plot2d"");a.children(1).children(2).mark_foreground = gcbo.value;", ...
    "tag", "popup2");
    frame = uicontrol("parent", btnFrame, ...
    "style", "frame", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "Polyline #3"), ...
    "constraints", createConstraints("gridbag", [1, 4, 1, 1]), ...
    "layout", "border");
    popup3 = uicontrol("parent", frame, ...
    "style", "popupmenu", ...
    "string", colors, ...
    "callback", "a=get(""plot2d"");a.children(1).children(3).mark_foreground = gcbo.value;", ...
    "tag", "popup3");

    plotFrame = uicontrol("parent", plotTab, ...
    "style", "frame", ...
    "constraints", createConstraints("border", "center"));
    a = newaxes(plotFrame);
    a.tag = "plot2d";
endfunction
plot2dInterations();
//==============================================================================
// Figure properties tab
//==============================================================================
function figureProperties()
    figureTab = uicontrol("parent", tab, ...
    "style", "frame",...
    "string", "Figure properties",...
    "layout", "gridbag");

    hiddenFrame = uicontrol("parent", figureTab, ...
    "style", "frame", ...
    "layout", "gridbag", ...
    "constraints", createConstraints("gridbag", [1, 1, 1, 1]));

    aboutFrame = uicontrol("parent", hiddenFrame, ...
    "style", "frame", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "About Scilab 5.5.0"), ...
    "layout", "border", ...
    "constraints", createConstraints("gridbag", [1, 1, 2, 1]));
    aboutScilab = uicontrol("parent", aboutFrame, ...
    "style", "image", ...
    "string", "aboutscilab.png");

    resizeFrame = uicontrol("parent", hiddenFrame, ...
    "style", "frame", ...
    "layout", "gridbag", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "Figure size"), ...
    "constraints", createConstraints("gridbag", [1, 2, 1, 1], [1 1], "horizontal"));
    radio1 = uicontrol("parent", resizeFrame, ...
    "style", "radiobutton", ...
    "group", "resizegroup", ...
    "string", "Resizable figure", ...
    "constraints", createConstraints("gridbag", [1, 1, 1, 1], [1 1], "both"), ...
    "callback", "set(gcf(), ""resize"", ""on"")");
    radio2 = uicontrol("parent", resizeFrame, ...
    "style", "radiobutton", ...
    "group", "resizegroup", ...
    "string", "Fixed size figure", ...
    "constraints", createConstraints("gridbag", [1, 2, 1, 1], [1 1], "both"), ...
    "callback", "set(gcf(), ""resize"", ""off"")", ...
    "horizontalalignment", "left")
    radio1.value = 1;

    iconFrame = uicontrol("parent", hiddenFrame, ...
    "style", "frame", ...
    "layout", "gridbag", ...
    "border", createBorder("titled", createBorder("etched", "lowered"), "Figure icon"), ...
    "constraints", createConstraints("gridbag", [2, 2, 1, 1], [1 1], "horizontal"));
    radio1 = uicontrol("parent", iconFrame, ...
    "style", "radiobutton", ...
    "group", "icongroup", ...
    "string", "Scilab icon", ...
    "constraints", createConstraints("gridbag", [1, 1, 1, 1], [1 1], "both"), ...
    "callback", "set(gcf(), ""icon"", ""scilab"")")
    radio2 = uicontrol("parent", iconFrame, ...
    "style", "radiobutton", ...
    "group", "icongroup", ...
    "string", "Customized icon", ...
    "constraints", createConstraints("gridbag", [1, 2, 1, 1], [1 1], "both"), ...
    "callback", "set(gcf(), ""icon"", ""applications-system"")")
    radio2.value = 1;
endfunction
figureProperties();
f.visible = "on";
clear f tab xcosTab plot2dInteractions figureProperties buttonsWithIcons matplotDemo matplotFrame
